# This program generates the LC ICP table for the paper (Table 2);
# it is based on table_ICP.R.

# preparing the data

start <- Sys.time()	# timing: start

epsrev = 20    # inverse significance level 1/epsilon

N <- 4601   # size of the data set
N.training <- 2602
N.calibration <- 999
N.test <- 1000
original.data <- read.csv("spam.data")

average.perc.err <- 0        # initializing the average percentage of errors overall
average.perc.err.email <- 0  # initializing the average percentage of email errors
average.perc.err.spam <- 0   # initializing the average percentage of spam errors

average.perc.mult <- 0       # initializing the average percentage of multiple predictions overall
average.perc.mult.email <- 0 # initializing the average percentage of multiple predictions for email
average.perc.mult.spam <- 0  # initializing the average percentage of multiple predictions for spam

average.perc.emp <- 0        # initializing the average percentage of empty predictions overall
average.perc.emp.email <- 0  # initializing the average percentage of empty predictions for email
average.perc.emp.spam <- 0   # initializing the average percentage of empty predictions for spam

N.columns <- 8     # in the paper: 8 (the number of colours in R minus white)

perc.err <- rep(0,N.columns)        # initializing the percentage of errors overall
perc.err.email <- rep(0,N.columns)  # initializing the percentage of email errors
perc.err.spam <- rep(0,N.columns)   # initializing the percentage of spam errors

perc.mult <- rep(0,N.columns)       # initializing the percentage of multiple predictions overall
perc.mult.email <- rep(0,N.columns) # initializing the percentage of multiple predictions for email
perc.mult.spam <- rep(0,N.columns)  # initializing the percentage of multiple predictions for spam

perc.emp <- rep(0,N.columns)        # initializing the percentage of empty predictions overall
perc.emp.email <- rep(0,N.columns)  # initializing the percentage of empty predictions for email
perc.emp.spam <- rep(0,N.columns)   # initializing the percentage of empty predictions for spam

cat("The CICP table for the batch.pdf paper", "\n\n", file="table_CICP.txt",append=FALSE)

for (column in 1:N.columns) {  # the first, say, 10 columns (seeds 0 to 9)
  set.seed(column-1)             # seeds 0 to 9

  permutation <- sample(1:N)
  permuted.data <- original.data[permutation,]
  rownames(permuted.data) <- 1:N
  training.set <- permuted.data[1:N.training,]
  calibration.set <- permuted.data[(N.training+1):(N.training+N.calibration),]
  rownames(calibration.set) <- 1:N.calibration
  test.set <- permuted.data[(N.training+N.calibration+1):(N.training+N.calibration+N.test),]
  rownames(test.set) <- 1:N.test

  # fit the model
  gbm1 <- gbm(V58~V1+V2+V3+V4+V5+V6+V7+V8+V9+V10+V11+V12+V13+V14+V15+V16+V17+V18+V19+V20+V21+V22+V23+V24+V25+V26+V27+V28+V29+V30+V31+V32+V33+V34+V35+V36+V37+V38+V39+V40+V41+V42+V43+V44+V45+V46+V47+V48+V49+V50+V51+V52+V53+V54+V55+V56+V57,
    data=training.set,
    distribution="bernoulli",    # adaboost or bernoulli
    n.trees=4000,                # number of trees
    shrinkage=0.005,             # shrinkage or learning rate,
                                 # 0.001 to 0.1 usually work
    interaction.depth=3,         # 1: additive model, 2: two-way interactions, etc.
    bag.fraction = 0.5,          # subsampling fraction, 0.5 is probably best
    train.fraction = 1.0,        # all data used for training
    n.minobsinnode = 10,         # minimum total weight needed in each node
    cv.folds = 5,                # do 5-fold cross-validation
    keep.data=TRUE)              # keep a copy of the dataset with the object

  # check performance using 5-fold cross-validation
  best.iter <- gbm.perf(gbm1,method="cv")
  # print(best.iter)

  # predict on the new data using "best" number of trees
  # .predict will be on the logit scale
  calibration.predict <- predict.gbm(gbm1,calibration.set,best.iter)
  test.predict <- predict.gbm(gbm1,test.set,best.iter)

  # conformity scores
  calibration.scores <- (2*calibration.set$V58-1)*calibration.predict
  test.scores <- (2*test.set$V58-1)*test.predict
  test.scores.spam <- 1*test.predict
  test.scores.email <- (-1)*test.predict

  calibration.labels <- calibration.set$V58  # vector of labels for the calibration set
  labels <- test.set$V58                     # vector of labels for the test set

  Err <- 0   # errors (overall)
  Mult <- 0  # multiple predictions (overall)
  Emp <- 0   # empty predictions (overall)

  Err.email <- 0  # errors for email only
  Mult.email <- 0 # multiple predictions for email only
  Emp.email <- 0  # empty predictions for email only

  Err.spam <- 0  # errors for spam only
  Mult.spam <- 0 # multiple predictions for spam only
  Emp.spam <- 0  # empty predictions for spam only

  N.email <- 0  # number of email in the test set
  N.spam <- 0   # number of spam in the test set

  for (i in 1:N.test) {
    rank.actual <- sum(as.numeric(calibration.scores[calibration.labels==labels[i]] <= test.scores[i]))
    rank.email <- sum(as.numeric(calibration.scores[calibration.labels==0] <= test.scores.email[i]))
    rank.spam <- sum(as.numeric(calibration.scores[calibration.labels==1] <= test.scores.spam[i]))

    N.calibration.actual <- sum(as.numeric(calibration.labels==labels[i]))
       # counting examples of the same class in the calibration set
    N.calibration.email <- sum(as.numeric(calibration.labels==0))  # counting email in the calibration set
    N.calibration.spam <- sum(as.numeric(calibration.labels==1))   # counting spam in the calibration set

    error <- ((rank.actual+1)*epsrev <= N.calibration.actual+1) # the condition is (rank+1)/(N+1) <= epsilon
    email <- ((rank.email+1)*epsrev > N.calibration.email+1)    # email is in the prediction set
    spam <- ((rank.spam+1)*epsrev > N.calibration.spam+1)       # spam is in the prediction set

    # counting errors, multiple predictions, empty predictions, spam, email, etc.
    if (error) {Err <- Err+1}
    if (spam & email) {Mult <- Mult+1}
    if ((!spam) & (!email)) {Emp <- Emp+1}
    label <- labels[i]
    if (label==1) {N.spam <- N.spam+1}
    if (label==0) {N.email <- N.email+1}
    if (error & (label==1)) {Err.spam <- Err.spam+1}
    if (spam & email & (label==1)) {Mult.spam <- Mult.spam+1}
    if ((!spam) & (!email) & (label==1)) {Emp.spam <- Emp.spam+1}
    if (error & (label==0)) {Err.email <- Err.email+1}
    if (spam & email & (label==0)) {Mult.email <- Mult.email+1}
    if ((!spam) & (!email) & (label==0)) {Emp.email <- Emp.email+1}
  }

  perc.err[column] <- Err/N.test
  perc.err.email[column] <- Err.email/N.email
  perc.err.spam[column] <- Err.spam/N.spam

  perc.mult[column] <- Mult/N.test
  perc.mult.email[column] <- Mult.email/N.email
  perc.mult.spam[column] <- Mult.spam/N.spam

  perc.emp[column] <- Emp/N.test
  perc.emp.email[column] <- Emp.email/N.email
  perc.emp.spam[column] <- Emp.spam/N.spam

  # Accumulating the averages
  average.perc.err <- average.perc.err + perc.err[column]/N.columns
  average.perc.err.email <- average.perc.err.email + perc.err.email[column]/N.columns
  average.perc.err.spam <- average.perc.err.spam + perc.err.spam[column]/N.columns
  average.perc.mult <- average.perc.mult + perc.mult[column]/N.columns
  average.perc.mult.email <- average.perc.mult.email + perc.mult.email[column]/N.columns
  average.perc.mult.spam <- average.perc.mult.spam + perc.mult.spam[column]/N.columns
  average.perc.emp <- average.perc.emp + perc.emp[column]/N.columns
  average.perc.emp.email <- average.perc.emp.email + perc.emp.email[column]/N.columns
  average.perc.emp.spam <- average.perc.emp.spam + perc.emp.spam[column]/N.columns
} # end of the big for

# printing out the statistics
for (column in 1:N.columns) {
  cat("Percentage of errors overall:", 100*perc.err[column], "\n", file="table_CICP.txt",append=TRUE)
  cat("Percentage of email errors:", 100*perc.err.email[column], "\n", file="table_CICP.txt",append=TRUE)
  cat("Percentage of spam errors:", 100*perc.err.spam[column], "\n", file="table_CICP.txt",append=TRUE)
  cat("Percentage of multiple predictions overall:", 100*perc.mult[column], "\n", file="table_CICP.txt",append=TRUE)
  cat("Percentage of email multiple predictions:", 100*perc.mult.email[column], "\n", file="table_CICP.txt",append=TRUE)
  cat("Percentage of spam multiple predictions:", 100*perc.mult.spam[column], "\n", file="table_CICP.txt",append=TRUE)
  cat("Percentage of empty predictions overall:", 100*perc.emp[column], "\n", file="table_CICP.txt",append=TRUE)
  cat("Percentage of email empty predictions:", 100*perc.emp.email[column], "\n", file="table_CICP.txt",append=TRUE)
  cat("Percentage of spam empty predictions:", 100*perc.emp.spam[column], "\n\n", file="table_CICP.txt",append=TRUE)
}

cat("Average percentage of errors overall:", 100*average.perc.err, "\n", file="table_CICP.txt",append=TRUE)
cat("Average percentage of email errors:", 100*average.perc.err.email, "\n", file="table_CICP.txt",append=TRUE)
cat("Average percentage of spam errors:", 100*average.perc.err.spam, "\n", file="table_CICP.txt",append=TRUE)
cat("Average percentage of multiple predictions overall:", 100*average.perc.mult, "\n", file="table_CICP.txt",append=TRUE)
cat("Average percentage of email multiple predictions:", 100*average.perc.mult.email, "\n", file="table_CICP.txt",append=TRUE)
cat("Average percentage of spam multiple predictions:", 100*average.perc.mult.spam, "\n", file="table_CICP.txt",append=TRUE)
cat("Average percentage of empty predictions overall:", 100*average.perc.emp, "\n", file="table_CICP.txt",append=TRUE)
cat("Average percentage of email empty predictions:", 100*average.perc.emp.email, "\n", file="table_CICP.txt",append=TRUE)
cat("Average percentage of spam empty predictions:", 100*average.perc.emp.spam, "\n\n", file="table_CICP.txt",append=TRUE)

end <- Sys.time()	# timing: end
cat("Finished. Time elapsed:", difftime(end,start), "\n")
